@@ -1,5 +1,6 @@
 """
 PrimalScheme: a primer3 wrapper for designing multiplex primer schemes
+
 Copyright (C) 2020 Joshua Quick and Andrew Smith
 www.github.com/aresti/primalscheme
 
@@ -25,12 +26,15 @@ import json
 import logging
 import pickle
 
+from abc import ABC, abstractmethod
 from collections import namedtuple
+
 from Bio import SeqIO
 from Bio.Graphics import GenomeDiagram
 from Bio.SeqFeature import FeatureLocation, SeqFeature
 from reportlab.lib import colors
 
+from primalscheme import config, __version__ as version
 from primalscheme.multiplex import MultiplexScheme
 from primalscheme.primer import Direction
 
@@ -41,20 +45,21 @@ Insert = namedtuple("Insert", "start end pool")
 
 
 class MultiplexReporter(MultiplexScheme):
-    """Reporting methods to extend MultiplexScheme"""
+    """Reporting methods to extend MultiplexScheme."""
 
     def __init__(self, outpath, *args, **kwargs):
+        """Init MultiplexReporter."""
         self.outpath = outpath
         super().__init__(*args, **kwargs)
 
     @property
     def inserts(self):
-        """A list of insert (start, end) tuples"""
+        """A list of insert (start, end) tuples."""
         return [Insert(r.left.end + 1, r.right.end - 1, r.pool) for r in self.regions]
 
     @property
     def gap_count(self):
-        """The total number of gaps in the scheme"""
+        """The total number of gaps in the scheme."""
         gaps = 0
         last_covered = self.inserts[0].end
         for insert in self.inserts[1:]:
@@ -65,20 +70,20 @@ class MultiplexReporter(MultiplexScheme):
 
     @property
     def percent_coverage(self):
-        """Coverage %, with respect to the primary reference"""
+        """Coverage %, with respect to the primary reference."""
         covered_coords = set(
             [x for insert in self.inserts for x in range(insert.start, insert.end + 1)]
         )
         return round(len(covered_coords) / self.ref_len * 100, 2)
 
     def calc_gc(self, sequence):
-        """Return gc content for a sequence as fraction"""
+        """Return gc content for a sequence as fraction."""
         g = sequence.count("G") + sequence.count("g")
         c = sequence.count("C") + sequence.count("c")
         return (g + c) / len(sequence)
 
     def write_default_outputs(self):
-        """Write all default output files"""
+        """Write all default output files."""
         self.write_primer_bed()
         self.write_insert_bed()
         self.write_primer_tsv()
@@ -87,7 +92,7 @@ class MultiplexReporter(MultiplexScheme):
         self.write_run_report_json()
 
     def write_primer_bed(self):
-        """Write primer BED file"""
+        """Write primer BED file."""
         filepath = self.outpath / f"{self.prefix}.primer.bed"
         logger.info(f"Writing {filepath}")
 
@@ -115,7 +120,7 @@ class MultiplexReporter(MultiplexScheme):
             cw.writerows(rows)
 
     def write_insert_bed(self):
-        """Write insert BED file"""
+        """Write insert BED file."""
         filepath = self.outpath / f"{self.prefix}.insert.bed"
         logger.info(f"Writing {filepath}")
 
@@ -139,7 +144,7 @@ class MultiplexReporter(MultiplexScheme):
             cw.writerows(rows)
 
     def write_primer_tsv(self):
-        """Write primer TSV file"""
+        """Write primer TSV file."""
         filepath = self.outpath / f"{self.prefix}.primer.tsv"
         logger.info(f"Writing {filepath}")
 
@@ -163,14 +168,14 @@ class MultiplexReporter(MultiplexScheme):
             cw.writerows(rows)
 
     def write_pickle(self):
-        """Write pickle file"""
+        """Write pickle file."""
         filepath = self.outpath / f"{self.prefix}.pickle"
         logger.info(f"Writing {filepath}")
         with open(filepath, "wb") as pickleobj:
             pickle.dump(self, pickleobj)
 
     def write_refs(self):
-        """Write reference FASTA"""
+        """Write reference FASTA."""
         filepath = self.outpath / f"{self.prefix}.reference.fasta"
         logger.info(f"Writing {filepath}")
         with open(filepath, "w"):
@@ -234,7 +239,7 @@ class MultiplexReporter(MultiplexScheme):
         return results  # Return the list of (position, value) results
 
     def write_schemadelica_plot(self):
-        """Write schemadelica plot as SVG and PDF"""
+        """Write schemadelica plot as SVG and PDF."""
         gd_diagram = GenomeDiagram.Diagram("Primer Scheme", track_size=0.15)
         primer_feature_set = GenomeDiagram.FeatureSet()
 
@@ -306,16 +311,89 @@ class MultiplexReporter(MultiplexScheme):
         gd_diagram.write(str(svg_filepath), "SVG", dpi=300)
 
     def write_run_report_json(self):
-        """Write run report json"""
+        """Write run report json."""
         filepath = self.outpath / f"{self.prefix}.report.json"
         logger.info(f"Writing {filepath}")
         data = {
             "references": [ref.id for ref in self.references],
             "primary_ref": self.references[0].id,
             "secondary_refs": [ref.id for ref in self.secondary_refs],
+            "secondary_ref_count": len(self.secondary_refs),
             "excluded_refs": [ref.id for ref in self.excluded_refs],
+            "excluded_ref_count": len(self.excluded_refs),
             "regions": len(self.regions),
             "percent_coverage": self.percent_coverage,
             "gaps": self.gap_count,
+            "config": {
+                "amplicon_size_min": self.amplicon_size_min,
+                "amplicon_size_max": self.amplicon_size_max,
+                "high_gc": self.high_gc,
+                "primalscheme_version": version,
+                "primary_only": self.primary_only,
+                "primer_size_range": {
+                    "min": config.PRIMER_SIZE_RANGE.min,
+                    "max": config.PRIMER_SIZE_RANGE.max,
+                    "opt": config.PRIMER_SIZE_RANGE.opt,
+                },
+                "primer_gc_range": {
+                    "min": config.PRIMER_GC_RANGE.min,
+                    "max": config.PRIMER_GC_RANGE.max,
+                    "opt": config.PRIMER_GC_RANGE.opt,
+                },
+                "step_distance": config.STEP_DISTANCE,
+                "target_overlap": self.target_overlap,
+            },
         }
         filepath.write_text(json.dumps(data))
+
+
+class ProgressTracker(ABC):
+    """Abstract base class for ProgressTracker."""
+
+    @abstractmethod
+    def goto(self, val):
+        """Update progress to val."""
+        ...
+
+    @property
+    def end(self):
+        """Progress end value."""
+        ...
+
+    @end.setter
+    @abstractmethod
+    def end(self, val):
+        """Set progress end value."""
+        ...
+
+    @property
+    def considered(self):
+        """Count of considered primers."""
+        ...
+
+    @considered.setter
+    @abstractmethod
+    def considered(self, considered):
+        """Set count of considered primers."""
+        ...
+
+    @property
+    def region_num(self):
+        """The current region num."""
+        ...
+
+    @region_num.setter
+    @abstractmethod
+    def region_num(self, region_num):
+        """Set current region num."""
+        ...
+
+    @abstractmethod
+    def interrupt(self):
+        """Prepare to be interrupted by a log message."""
+        ...
+
+    @abstractmethod
+    def finish(self):
+        """Finish tracking progress."""
+        ...
